#include <stdio.h>
#include "raylib.h"

#define SCREEN_MIDDLE ((Vector2) {window_size.x/2, window_size.y/2})

typedef struct Player {
    float radius;
    Vector2 coord;
} Player;

typedef struct Map {
    const Texture texture;
    const Rectangle box;
} Map;

Camera2D shake(Camera2D cam, int force)
{
    cam.offset.x += GetRandomValue(-force/2, force/2);
    cam.offset.y += GetRandomValue(-force/2, force/2);
    return cam;
}

int main(void)
{
    Vector2 window_size = { 800.0f, 600.0f };

    Camera2D camera = {
        .target = {0},
        .offset = SCREEN_MIDDLE,
        .rotation = 0.0f,
        .zoom = 1.0f,
    };

    Player player = {
        .radius = 25.0f,
    };

#ifdef RELEASE
    SetTraceLogLevel(LOG_FATAL);
#endif

    InitWindow(window_size.x, window_size.y, "voleur");
    SetWindowState(FLAG_WINDOW_RESIZABLE);
    SetTargetFPS(60);

    Map map = {
        .texture = LoadTexture("data/map2.png"),
        .box = (Rectangle) {
            .x = -map.texture.width/2, .y = -map.texture.height/2,
            .width = map.texture.width, .height = map.texture.height,
        },
    };

    int keyboard_layout_azerty = 1;

    while (!WindowShouldClose()) {
        float DT = GetFrameTime();
        if (IsWindowResized()) {
            window_size = (Vector2) {GetScreenWidth(), GetScreenHeight()};
            camera.offset = SCREEN_MIDDLE;
        }

        float movement_speed = 10.0f * DT * 50;
        if ((IsKeyDown(KEY_LEFT) || IsKeyDown(KEY_RIGHT)) && (IsKeyDown(KEY_UP) || IsKeyDown(KEY_DOWN)))
            movement_speed = 6.324f * DT * 50;
        if (IsKeyDown(KEY_LEFT_SHIFT))
            movement_speed /= 2.0f;

        if (keyboard_layout_azerty) {
            if (IsKeyDown(KEY_A))
                camera.target.x -= movement_speed ;
            if (IsKeyDown(KEY_D))
                camera.target.x += movement_speed ;
            if (IsKeyDown(KEY_W))
                camera.target.y -= movement_speed ;
            if (IsKeyDown(KEY_S))
                camera.target.y += movement_speed ;
        } else {
            if (IsKeyDown(KEY_Q))
                camera.target.x -= movement_speed ;
            if (IsKeyDown(KEY_D))
                camera.target.x += movement_speed ;
            if (IsKeyDown(KEY_Z))
                camera.target.y -= movement_speed ;
            if (IsKeyDown(KEY_S))
                camera.target.y += movement_speed ;
        }

        if (IsKeyReleased(KEY_F))
            camera.target = player.coord;
        if (!IsKeyDown(KEY_F)) {
            player.coord = GetScreenToWorld2D(SCREEN_MIDDLE, camera);

            if (player.coord.x - player.radius < map.box.x)
                camera.target.x = map.box.x + player.radius;
            else if (player.coord.x + player.radius > map.box.x + map.box.width)
                camera.target.x = map.box.x + map.box.width - player.radius;

            if (player.coord.y - player.radius < map.box.y)
                camera.target.y = map.box.y + player.radius;
            else if (player.coord.y + player.radius > map.box.y + map.box.height)
                camera.target.y = map.box.y + map.box.height - player.radius;

            player.coord = GetScreenToWorld2D(SCREEN_MIDDLE, camera);
        }

        Camera2D effect_camera = camera;
        if (IsKeyDown(KEY_SPACE))
            effect_camera = shake(camera, 12);

        BeginDrawing();
        {
            ClearBackground(LIME);

            BeginMode2D(effect_camera);
            {
                DrawTexture(map.texture, map.box.x, map.box.y, WHITE);
                DrawCircle(player.coord.x, player.coord.y, player.radius, RED);
            }
            EndMode2D();
        }
        EndDrawing();
    }

    CloseWindow();
    return 0;
}
